#include "mcc_generated_files/mcc.h"
#include "mcc_generated_files/examples/i2c1_master_example.h"

#define SLAVE_ADRS 0x3E
#define LINE1_ADRS 0x40
#define LINE2_TOP (0x40 +0x80)
#define FUNC1_SET 0x38
#define FUNC2_SET 0x39
#define INT_OSC 0x14
#define CONST_SET 0x77
#define PWR_ICON_SET 0x54 //0x56
#define FOLLOWER_SET 0x6C
#define CLR_DISP 0x01
#define DISP_ONOFF_SET 0x0C

#define TIME_OFF -1         //タイマーを使用しない場合
#define TIME_UP 0           //タイムアップ
#define TIME_BASE_MAX 10     //ベースタイマカウント値
#define LCD_TIM_MAX 1
#define TIME_FILTER_MAX 1
#define	CNT_INIT_MAX 10     //10ms×10 = 100ms
#define WATCH_CNT_MAX 200
#define DI_NUM 2            //DIフィルタのサンプリング数

typedef struct{
    uint8_t    wp;
    uint8_t    buf[ DI_NUM ];
    uint8_t    di;
}FILT_DATA;

/* 変数定義 */
int16_t cnt10ms;
uint8_t CntInit; //初期化時のみ使用
uint32_t timcnt;
int8_t timlcd;
int16_t timfilter; //DIフィルタ起動
int16_t timcntclear = TIME_OFF;
FILT_DATA DiData;
bool startwatch = false;
bool btn1hold;
uint8_t initmoji[2][16] ={"PIC16F1827-I2C","         Ver1.00"};
uint8_t disptbl1[2][16] ={"STOP WATCH    ","                "};

/* 関数ﾌﾟﾛﾄﾀｲﾌﾟ宣言 */
void LcdInit(void);
void DspLine2Top(void);
void DspClear(void);
void mainTimer(void);
void Tmr0Int(void);
void DispSet(void);
void ShowDataSet(void);
void DiFilter(void);
void DiFilterInit(void);
void mainApp(void);

/* Main application */
void main(void)
{
    SYSTEM_Initialize();
    TMR0_SetInterruptHandler(mainTimer);
    INTERRUPT_GlobalInterruptEnable();
    INTERRUPT_PeripheralInterruptEnable();
    DiFilterInit(); //メイン処理前にDIを確定する
    LcdInit();
    
    while (1)
    {
        mainApp();
        DiFilter();
    }
}
/* メイン関数 */
void mainApp(void){
    
    if( DiData.di == 0){
        if( btn1hold == false ){
            btn1hold = true;
            timcntclear = WATCH_CNT_MAX;
            
            if( startwatch){
                startwatch = false;
            } 
            else{
                startwatch = true;
            }
        }
    }
    else{
        btn1hold = false;
        timcntclear = TIME_OFF;
    }    
 
    if( timlcd == TIME_UP ){
        timlcd = LCD_TIM_MAX;
        DispSet(); //LCD表示部分をセット
    }
    
    if( timcntclear == TIME_UP ){
        timcntclear = TIME_OFF;
        startwatch = false;
        timcnt = 0;
    }
}
/* タイマ管理関数 */
void mainTimer(void){
  
    if( timlcd > TIME_UP ){
        --timlcd;
    }
    
    if( timfilter > TIME_UP ){
        --timfilter; //タイマを更新
    }
    
    if( timcntclear > TIME_UP ){
        --timcntclear;
    }

    if( startwatch && timcntclear == TIME_OFF ){
        ++timcnt;
    }
}
/* DIフィルタの初期化 */
void DiFilterInit(void){
	
    CntInit = CNT_INIT_MAX;
    while(CntInit > 0){  //0になるまでフィルタを実施
        DiFilter();         //DIフィルタ処理
	__delay_ms(10);     //10ms遅延させてDIフィルタ処理
        CntInit--;
	timfilter = TIME_UP;
    }    
}
/* モード切替部のDIフィルタ */
void DiFilter(void){
    uint8_t i;
    bool boo = true;

    if( timfilter == TIME_UP ){
        timfilter = TIME_FILTER_MAX;
        DiData.buf[ DiData.wp ] = IO_RB0_GetValue();

        for( i = 1; i < DI_NUM; i++){
            if( DiData.buf[i-1] != DiData.buf[i] ){
                boo = false;
                break;
            }
        }

        if( boo ){ //比較して一致したら値を採用
             DiData.di = DiData.buf[ 0 ]; 
        }

        if( ++DiData.wp >= DI_NUM){
                DiData.wp = 0;
        }
    }
}
/* LCDの初期化 */
void LcdInit(void){
    uint8_t i;
    
    I2C1_Write1ByteRegister(SLAVE_ADRS, 0x00, FUNC1_SET ); //８ビットバス・2LINE表示
    __delay_us(40);
    I2C1_Write1ByteRegister(SLAVE_ADRS, 0x00, FUNC2_SET ); //拡張コマンド
    __delay_us(40);
    I2C1_Write1ByteRegister(SLAVE_ADRS, 0x00, INT_OSC ); //内部周波数調整
    __delay_us(40);    
    I2C1_Write1ByteRegister(SLAVE_ADRS, 0x00, CONST_SET ); //コントラスト1
    __delay_us(40);    
    I2C1_Write1ByteRegister(SLAVE_ADRS, 0x00, PWR_ICON_SET );//コントラスト2
    __delay_us(40);   
    I2C1_Write1ByteRegister(SLAVE_ADRS, 0x00, FOLLOWER_SET );//フォロワー制御
    __delay_ms(250);
    I2C1_Write1ByteRegister(SLAVE_ADRS, 0x00, FUNC1_SET ); //拡張コマンドをオフ
    __delay_us(40);
    
    DspClear(); //LCD表示をクリア
    
    I2C1_Write1ByteRegister(SLAVE_ADRS, 0x00, DISP_ONOFF_SET );
    __delay_us(40);
    
    //for( i = 0; i < sizeof(initmoji[0]); i++ ){
    //    I2C1_Write1ByteRegister(SLAVE_ADRS,LINE1_ADRS,initmoji[0][i]);
    //}
      
    I2C1_WriteNBytes( SLAVE_ADRS,&initmoji[0][0], sizeof(initmoji[0])); //LINE1の表示（上のforと同じ）
    
    DspLine2Top(); //カーソルをLINE2の先頭にする

    for( i = 0; i < sizeof(initmoji[1]); i++ ){
        I2C1_Write1ByteRegister(SLAVE_ADRS,LINE1_ADRS,initmoji[1][i]);　//LINE2の表示
    }        
    __delay_ms(2000);
    DspClear();
    
    for( i = 0; i < sizeof(disptbl1[0]); i++ ){
        I2C1_Write1ByteRegister(SLAVE_ADRS,LINE1_ADRS,disptbl1[0][i]); //LINE1を表示
    }     
}
/* LINE2へのカーソル移動処理*/
void DspLine2Top(void){
    
    I2C1_Write1ByteRegister(SLAVE_ADRS, 0x00, LINE2_TOP );
    __delay_us(40);    
}
/* ディスプレイをクリア */
void DspClear(void){
  
    I2C1_Write1ByteRegister(SLAVE_ADRS, 0x00, CLR_DISP );
    __delay_us(40);
}
/* LINE2の表示（ストップウォッチ） */
void DispSet(void){
    uint8_t i;
    
    DspLine2Top();
    ShowDataSet();
    
    for( i = 0; i < sizeof(disptbl1[1]); i++ ){
        I2C1_Write1ByteRegister(SLAVE_ADRS,LINE1_ADRS,disptbl1[1][i]); //LINE2の表示（ストップウォッチ）
    } 
}
/* ストップウォッチの表示 */
void ShowDataSet(void){
    uint8_t msdata;
    uint8_t sdata;
    uint32_t mdata;
    uint8_t ms10;
    uint8_t ms1;
    uint8_t s10;
    uint8_t s1;
    uint32_t m10;
    uint8_t m1;
    
    mdata = (timcnt / 100 ) / 60;
    sdata = (timcnt / 100 ) % 60;
    msdata = timcnt % 100;
    
    ms10 = msdata / 10 + 0x30;
    ms1 = msdata  % 10 + 0x30;
    s10 = sdata / 10 + 0x30;
    s1 = sdata % 10 + 0x30;
    m10 = mdata / 10 + 0x30;
    m1 = mdata % 10 + 0x30;
    
    disptbl1[1][8] = (uint8_t)m10;
    disptbl1[1][9] = m1;    
    disptbl1[1][10] = ':';
    disptbl1[1][11] = s10;
    disptbl1[1][12] = s1;
    disptbl1[1][13] = '.';
    disptbl1[1][14] = ms10;
    disptbl1[1][15] = ms1;
}
/**
 End of File
*/